;;; -*- Mode: Scheme; scheme48-package: swank-networking -*-

;;;; SLIME for Scheme48
;;;; Network Interface

;;; This code is written by Taylor R. Campbell and placed in the Public
;;; Domain.  All warranties are disclaimed.

(run-swank-session-server world
  (lambda ()                            ;acceptor
    (receive (in out) (socket-accept socket)
      (values in out (lambda ()
                       (close-input-port in)
                       (close-output-port out)))))
  (lambda (in out continuation)         ;negotiator
    in out
    (continuation))
  (lambda (session-placeholder body)    ;session wrapper
    (with-sldb-handler #f
      (lambda ()
        ((lambda (thunk)
           (if exit-on-quit?
               (with-swank-quitter
                   (lambda ()
                     (scheme-exit-now 0))
                 thunk)
               (thunk)))
         (lambda ()
           (with-slime48-port-redirection
               session-placehlder
             body)))))))

(define (serve-one-swank-session session connector negotiator)
  (let loop ()
    (receive (disconnector . arguments) (connector)
      (apply negotiator
             (lambda ()
               (set-swank-session-d)
               (let ((detacher
                      (attach-swank-session-i/o session in out
                                                detacher)))
                 (wrapper (lambda ()
                            ))))))))

(define (run-swank-session-server world acceptor negotiator wrapper)
  (let loop ()
    (receive (in out detacher) (acceptor)
      (spawn (lambda ()
               (negotiator
                in out
                (lambda arguments
                  (spawn-swank-session-with-i/o world wrapper
                                                in out detacher
                                                arguments))
                values))
             `(SWANK-NEGOTIATION ,in ,out)))
    (loop)))

(define (spawn-swank-session-with-i/o world wrapper in out detacher
                                      arguments)
  (let ((session-placeholder (make-placeholder)))
    (apply wrapper
           session-placeholder
           (lambda (winder unwinder)
             (let* ((session
                     (spawn-swank-session world winder unwinder)))
               (set-swank-session-i/o-detacher!
                session
                (attach-swank-session-i/o session in out))
               (placeholder-set! session-placeholder session)))
           arguments)))

(define (attach-swank-session-i/o session input-port output-port
                                  detacher)
  (cond ((swank-session-i/o-detacher session)
         => (lambda (detacher)
              (warn "replacing Swank session I/O" session)
              (detacher)
              (set-swank-session-i/o-detacher! session #f))))
  (let ((reader (spawn (lambda ()
                         (run-swank-reader session input-port))
                       `(SWANK-READER ,session ,input-port)))
        (writer (spawn (lambda ()
                         (run-swank-writer session output-port))
                       `(SWANK-WRITER ,session ,output-port))))
    (lambda ()
      (detacher)
      (terminate-thread! reader)
      (terminate-thread! writer))))

(define (detach-swank-session-i/o session)
  (cond ((swank-session-i/o-detacher session)
         => (lambda (detacher)
              (detacher)
              (set-swank-session-i/o-detacher! session #f)))))

(define (run-swank-reader session input-port)
  (let loop ()
    (send-incoming-swank-message session
                                 (decode-swank-message input-port))
    (loop)))

(define (run-swank-writer session output-port)
  (let loop ()
    ()))
