;;; -*- mode: scheme; scheme48-package: (exec) -*-

;;;;;; SLIME for Scheme48
;;;;;; Convenient load script, for testing

;;; This code is written by Taylor Campbell and placed in the Public
;;; Domain.  All warranties are disclaimed.

(in 'debugging
    '(run (define (breakpoint . args)
            (command-loop (make-condition 'breakpoint args)))))
(config '(open list-interfaces))

(translate "=s48util/"
           "~/programming/scheme/scheme48/utilities/")
(config '(load "=s48util/defrectype*.scm"
               "=s48util/interface-doc.scm"))

(in 'debugging '(run (define (breakpoint msg . args)
                       (command-loop (cons 'breakpoint
                                           (cons msg args))))))

(exec '(open posix-files))              ;++ crock
(exec '(run (config `(load ,@(map (lambda (fname)
                                    (string-append (working-directory)
                                                   "/" fname))
                                  '("interfaces.scm"
                                    "packages.scm"))))))
(user '(open swank-worlds swank-sessions swank-sldb swank-tcp-servers
             receiving))
(load-package 'swank-rpc)

(map (lambda (s) (in s '(open debugging)))
     '(swank-sessions swank-sldb swank-tcp-servers))

(exec '(open environments package-commands-internal))
(exec '(run (user (lambda ()
                    (for-each (lambda (spec)
                                (environment-define!
                                    (interaction-environment)
                                    (car spec)
                                  (environment-ref (config-package)
                                                   (cadr spec))))
                              '((scheme scheme)
                                (config-lang module-system)
                                (built-in built-in-structures)
                                (rpcs swank-rpc)))))))

(user '(open package-commands-internal))
(user '(run
        (define world
          (receive (scratch config rpc)
                   (make-swank-envs scheme config-lang built-in rpcs)
            (make-swank-world scratch
                              (config-package)
                              rpc
                              'slate)))))

;;; Get rid of Scheme48's crap of a SRFI 35 implementation.

(new-package)
(open 'exceptions-internal 'vm-exceptions)
(run '(begin (initialize-vm-exceptions! really-signal-condition)
             #t)) ; Don't print what INITIALIZE-VM-EXCEPTIONS! returns.
(user)
