;;; -*- mode: scheme; scheme48-package: swank-pp -*-

;;;;;; SLIME for Scheme48
;;;;;; Pretty-printer

;;; This code is written by Taylor Campbell and placed in the Public
;;; Domain.  All warranties are disclaimed.

(define-record-type* pp-node
  (make-pp-node dimensions-fn position-fn printer)
  ())

(define (pp-node-dimensions node)
  ((pp-node-dimensions-fn node)))

(define (pp-node-position node start limit)
  ((pp-node-position-fn node) start limit))

(define (print-pp-node node start port)
  ((pp-node-printer node) start port))

(define (make-pp-leaf width obj nesting)
  (let ((width (+ width (length nesting))))
    (make-pp-node (lambda () width)
                  (lambda (start limit)
                    (let ((right (+ start width)))
                      (and (< right limit)
                           right)))
                  (lambda (start port)
                    (write obj port)
                    (for-each (lambda (close)
                                (write-char close port))
                              nesting)))))

(define (make-prefixed-pp-node prefix node)
  (let ((len (string-length prefix)))
    (make-pp-node (lambda () (+ len (pp-node-dimensions node)))
                  (lambda (start limit)
                    (pp-node-position node (+ start len) limit))
                  (lambda (start port)
                    (write-string prefix port)
                    (print-pp-node node (+ start len) port)))))



(define (simple-pp-prescan obj nesting recur)
  (cond ((symbol?  obj) (pp-prescan-symbol obj nesting))
        ((number?  obj) (pp-prescan-number obj nesting))
        ((boolean? obj) (make-pp-leaf 2    obj nesting))
        ((string?  obj) (pp-prescan-string obj nesting))
        ((char?    obj) (pp-prescan-char   obj nesting))
        ((null?    obj) (make-pp-leaf 2    obj nesting))
        ((vector?  obj) (pp-prescan-vector obj nesting recur))
        (else #f)))

(define (pp-prescan-symbol symbol nesting)
  (make-leaf (string-length (symbol->string symbol))
             symbol
             nesting))

(define (pp-prescan-number number nesting)
  (make-leaf (string-length (number->string number))
             number
             nesting))

(define (pp-prescan-string string nesting)
  (let ((len (string-length string)))
    (let loop ((i 0) (print-chars 0))
      (if (= i len)
          (make-leaf print-chars string nesting)
          (loop (+ i 1)
                (+ print-chars
                   (string-char-printed-length
                    (string-ref string i))))))))

(define (pp-prescan-char char nesting)
  (make-leaf (+ 2 (char-name-printed-length char))
             char
             nesting))

(define (string-char-printed-length char)
  (if (or (char=? char #\\)
          (char=? char #\"))
      2
      1))

(define (char-name-printed-length char)
  (case char
    ((#\newline) 7)
    ((#\space)   5)
    (else        1)))

(define (pp-prescan-vector vector nesting recur)
  (let* ((len (vector-length vector))
         (prescan-vector (vector-map (lambda (elt)
                                       (recur elt (cons #\) nesting)))
                                     vector)))
    (make-pp-node (lambda ()
                    (do ((i 0 (+ i 1))
                         (d 0 (+ d (pp-node-dimensions
                                    (vector-ref prescan-vector i)))))
                        ((= i len) d)))
                  (lambda (start limit)
                    (let loop ((i 0) (column (+ start 2)))
                      )
                    (do ((i 0 (+ i 1))
                         (c (+ start 2)
                            )))))))


