(define-interface swank-repl-interface
  (export repl-eval-string
          interactive-eval-results
          pprint-eval-results
          delimited-object-list-string  ; This should go elsewhere.
          swank-repl-presentations?
          enable-swank-repl-presentations
          disable-swank-repl-presentations
          toggle-swank-repl-presentations
          ))

(define-interface swank-sldb-interface
  (export with-sldb-handler
          sldb-condition-handler
          send-sldb-activation
          send-sldb-return
          eval-in-sldb-frame
          ))

(define-interface string-i/o-interface
  (export make-string-input-port
          make-string-output-port
          string-output-port-output
          call-with-string-output-port
          with-output-to-string
          read-from-string
          display-to-string
          write-to-string
          pp-to-string
          print-to-string
          print-limited-to-string
          print-shared-to-string
          print-hybrid-to-string
          ))

(define-interface printing-interface
  (export
    print
    $print-depth
    $print-breadth
    print-depth
    print-breadth
    set-print-depth!
    set-print-breadth!
    with-printing-limits
    $print-limited?
    $print-shared?
    print-limited?
    print-shared?
    set-print-limited?!
    set-print-shared?!
    with-limited-printing
    with-shared-printing
    with-hybrid-printing
    with-preserved-printing
    ))

(define-structure string-i/o string-i/o-interface
  (open scheme
        extended-ports
        (subset i/o-internal (call-with-current-output-port))
        pp
        printing
        )
  (optimize auto-integrate)
  (begin (define (with-output-to-string thunk)
           (call-with-string-output-port
             (lambda (port)
               (call-with-current-output-port port
                 thunk))))
         (define (read-from-string string)
           (read (make-string-input-port string)))
         (define (to-string obj printer)
           (call-with-string-output-port
             (lambda (port)
               (printer obj port))))
         (define (display-to-string obj) (to-string obj display))
         (define (write-to-string   obj) (to-string obj write))
         (define (pp-to-string      obj) (to-string obj p))
         (define (print-to-string   obj) (to-string obj print))
         (define (print-limited-to-string obj)
           (to-string obj print-limited))
         (define (print-shared-to-string obj)
           (to-string obj print-shared))
         (define (print-hybrid-to-string obj)
           (to-string obj print-hybrid))
         ))

(define-structure printing printing-interface
  (open scheme
        fluids
        cells
        writing
        simple-signals
        (subset i/o (make-null-output-port write-string))
        )
  (optimize auto-integrate)
  (files print))
